@@ -13,14 +13,12 @@ import sys
 
 from squyrrel.core.registry.exceptions import *
 from squyrrel.core.registry.exception_handler import ExceptionHandler
-from squyrrel.core.registry.meta import PackageMeta, ClassMeta
-from squyrrel.core.registry.signals import (squyrrel_debug_signal, squyrrel_error_signal,
-    class_loaded_signal)
+from squyrrel.core.registry.meta import PackageMeta, ClassMeta, ModuleMeta
+from squyrrel.core.registry.signals import (squyrrel_debug_signal, class_loaded_signal) # squyrrel_error_signal
 from squyrrel.core.utils.singleton import Singleton
 from squyrrel.core.utils.paths import convert_path_to_import_string, find_first_parent
 from squyrrel.core.config.base import ConfigRegistry, IConfig
 from squyrrel.core.config.decorators import exclude_from_logging
-from squyrrel.core.logging.utils import arguments_tostring
 
 
 __SQUYRREL_PACKAGE_NAME__ = 'squyrrel'
@@ -37,7 +35,7 @@ class Squyrrel(metaclass=Singleton):
 
         self.load_squyrrel_package()
 
-        print(f'Squyrrel ({id(self)}) awakened')
+        self.debug(f'Squyrrel ({id(self)}) awakened')
 
     def initialize(self, root_path=None, config_class=None):
         self.packages = {}
@@ -187,8 +185,8 @@ Packages not loaded (because they were filtered): {', '.join(filtered_packages)}
         paths = '\n'.join(paths_tried)
         self.debug('Did not find package <{relative_path}>. Tried the following paths: \n{paths}'.format(
             relative_path=relative_path, paths=paths))
-        print('sys.path:')
-        print(sys.path)
+        # print('sys.path:')
+        # print(sys.path)
         return None
 
     def find_package_by_name(self, name):
@@ -205,6 +203,7 @@ Packages not loaded (because they were filtered): {', '.join(filtered_packages)}
             return self.register_package(name)
 
     def inspect_directory(self, package_meta):
+        # todo: write test, what about sub_dirs?
         modules = []
         for root, sub_dirs, files in os.walk(package_meta.path):
             for file in files:
@@ -249,8 +248,8 @@ Packages not loaded (because they were filtered): {', '.join(filtered_packages)}
             imported_module = importlib.import_module(module_meta.import_string)#, package=package.import_string)
         except ModuleNotFoundError:
             module_meta.status = 'not found'
-            print('package:', package.import_string)
-            print('module:', module_meta.import_string)
+            # print('package:', package.import_string)
+            # print('module:', module_meta.import_string)
 
             # imported_module = importlib.import_module('.'+module_name, package=package.import_string)
 
@@ -354,7 +353,6 @@ Packages not loaded (because they were filtered): {', '.join(filtered_packages)}
             register_package_filter = self._register_package_filter
         if not register_package_filter(package_name):
             self.debug(f'Package {package_name} did not pass register filter')
-            print(f'Package {package_name} did not pass register filter')
             package_meta.status = 'excluded by filter'
             return package_meta
 
@@ -368,12 +366,12 @@ Packages not loaded (because they were filtered): {', '.join(filtered_packages)}
     @property
     def loaded_packages(self):
         return [package for package in self.packages if package.loaded]
-        return self._loaded_packages
 
     def load_package(self, package_meta, ignore_rotten_modules=False,
                            load_classes=True, load_subpackages=True,
                            load_package_filter=None, raise_when_already_loaded=False):
-        self.debug('Load package <{package}>...'.format(package=repr(package_meta)))
+        msg = 'Load package <{package}>...'.format(package=repr(package_meta))
+        self.debug(msg)
 
         if package_meta.loaded:
             if raise_when_already_loaded:
@@ -387,7 +385,6 @@ Packages not loaded (because they were filtered): {', '.join(filtered_packages)}
             load_package_filter = self._load_package_filter
         if not load_package_filter(package_meta):
             self.debug(f'Package {str(package_meta)} did not pass loading filter')
-            print(f'Package {str(package_meta)} did not pass loading filter')
             package_meta.status = 'excluded by filter'
             return package_meta
 
@@ -471,12 +468,16 @@ Packages not loaded (because they were filtered): {', '.join(filtered_packages)}
     def get_object(self, class_name_or_meta, package_name=None, module_name=None,
                     create_instance_when_not_found=True, params=None):
         # todo: refine searching criteria
-        print('get_object:', class_name_or_meta)
+        self.debug('get_object:' + str(class_name_or_meta))
 
         if isinstance(class_name_or_meta, ClassMeta):
             class_meta = class_name_or_meta
         else:
-            class_meta = self.find_class_meta_by_name(class_name, package_name=package_name, module_name=module_name)
+            class_meta = self.find_class_meta_by_name(
+                class_name_or_meta,
+                package_name=package_name,
+                module_name=module_name
+            )
         try:
             return class_meta.get_first_instance()
         except IndexError:
@@ -497,4 +498,4 @@ Packages not loaded (because they were filtered): {', '.join(filtered_packages)}
 
     def load_orm_packages(self):
         for package in ('sql', 'db', 'orm'):
-            self.register_and_load_package(package)
\ No newline at end of file
+            self.register_and_load_package(package)
