@@ -3,6 +3,8 @@
 
 class SqlDatabaseConnection:
 
+    database_error_cls = None
+
     def __init__(self):
         self.c = None
         self._cursor = None
@@ -11,9 +13,22 @@ class SqlDatabaseConnection:
         self._cursor = self.c.cursor(*args, **kwargs)
         return self._cursor
 
+    def close_cursor():
+        if self._cursor is not None:
+            self._cursor.close()
+            self._cursor = None
+
     def connect(self, **kwargs):
         raise NotImplementedError
 
+    def close_connection(self):
+        if self.c:
+            self.c.close()
+
+    def close(self):
+        self.close_cursor()
+        self.close_connection()
+
     def execute(self, sql, cursor=None, params=None):
         if cursor is not None:
             self._cursor = cursor
@@ -40,4 +55,4 @@ class SqlDatabaseConnection:
         if self._cursor is None:
             raise Exception('Cursor is None')
         data = self._cursor.fetchone()
-        return data
\ No newline at end of file
+        return data
