@@ -31,4 +31,8 @@ class RelationNotFoundException(Exception):
             msg = 'Did not find relation'
         super().__init__(msg)
         self.fk_id_column = fk_id_column
-        self.foreign_model = foreign_model
\ No newline at end of file
+        self.foreign_model = foreign_model
+
+
+class SqlException(Exception):
+    pass
\ No newline at end of file
