@@ -2,14 +2,13 @@ from squyrrel.sql.query import (Query, UpdateQuery, InsertQuery,
     DeleteQuery, CreateTableQuery)
 from squyrrel.sql.clauses import *
 from squyrrel.sql.expressions import (Equals, NumericalLiteral,
-    StringLiteral, Like, And, Or, Parameter)
-from squyrrel.sql.references import ColumnReference, TableReference
+    StringLiteral, And, Parameter)
+from squyrrel.sql.references import ColumnReference
 from squyrrel.sql.join import OnJoinCondition, JoinConstruct, JoinType
 from squyrrel.orm.exceptions import *
 from squyrrel.orm.field import (ManyToOne, ManyToMany, StringField,
     DateTimeField, IntegerField)
-from squyrrel.orm.filter import (ManyToOneFilter, ManyToManyFilter,
-    StringFieldFilter)
+from squyrrel.orm.filter import (ManyToOneFilter, ManyToManyFilter)
 from squyrrel.orm.signals import model_loaded_signal
 from squyrrel.orm.utils import extract_ids
 from squyrrel.orm.query_builder import QueryBuilder
@@ -25,37 +24,51 @@ class QueryWizzard:
         model_loaded_signal.connect(self.on_model_loaded)
 
     def commit(self):
-        print('COMMIT')
+        #print('COMMIT')
         self.db.commit()
 
     def rollback(self):
-        print('ROLLBACK')
+        #print('ROLLBACK')
         self.db.rollback()
 
-    def execute_query(self, query):
-        # todo: log!
-        sql = self.builder.build(query)
-        print('\n'+sql)
-        print('params:', query.params)
-        self.last_sql_query = query
+    def last_insert_rowid(self):
+        # todo: this only implements sqlite
+        # other like postgres..
+
+        self.execute_sql(sql="SELECT last_insert_rowid()")
+
+        data = self.db.fetchone()
+        if not data:
+            return None
+        return data[0]
+
+    def execute_sql(self, sql, params=None):
         try:
-            self.db.execute(sql=sql, params=query.params)
+            self.db.execute(sql=sql, params=params)
         except Exception as exc:
             # todo: log
             raise self.sql_exc(sql, exc) from exc
 
+    def execute_query(self, query):
+        # todo: log!
+        sql = self.builder.build(query)
+        # print('\n'+sql)
+        # print('params:', query.params)
+        self.last_sql_query = query
+        self.execute_sql(sql, params=query.params)
+
     def execute_queries_in_transaction(self, queries):
-        print(f'start transaction, {len(queries)} queries')
+        # print(f'start transaction, {len(queries)} queries')
         try:
             for query in queries:
-                print('execute query with params', query.params)
+                # print('execute query with params', query.params)
                 self.execute_query(query)
         except Exception as exc:
             self.rollback()
-            raise self.sql_exc(sql, exc) from exc
+            raise exc
         else:
             self.commit()
-            print('successfully committed all queries in transaction')
+            # print('successfully committed all queries in transaction')
 
     def on_model_loaded(self, *args, **kwargs):
         new_model_class_meta = kwargs.get('class_meta') or args[0]
@@ -66,14 +79,14 @@ class QueryWizzard:
 
     def register_model(self, model_cls_meta, table_name):
         if table_name is None:
-            print(f'Warning: Model {model_cls_meta.class_name} has table_name=None. Will not be registered.')
+            # print(f'Warning: Model {model_cls_meta.class_name} has table_name=None. Will not be registered.')
             return
         key = model_cls_meta.class_name
         if key in self.models.keys():
-            print(f'There is already a model on key <{key}>')
+            # print(f'There is already a model on key <{key}>')
             return
         self.models[key] = model_cls_meta.class_reference
-        print('register_model:', key)
+        # print('register_model:', key)
 
     def get_model(self, model):
         if isinstance(model, str):
@@ -91,7 +104,8 @@ class QueryWizzard:
         return None
 
     def sql_exc(self, sql, exc):
-        return Exception(f'Error during execution of query: \n{sql}\nSql Exc.: {str(exc)}')
+        error_category = 'Sql Error'
+        return SqlException(f'Error during execution of query: \n{sql}\n{error_category}: {str(exc)}')
 
     def build_select_fields(self, model, select_fields=None):
         if select_fields is None:
@@ -276,7 +290,7 @@ class QueryWizzard:
                 page_size = options.get('page_size', None)
                 active_page = options.get('active_page', None)
 
-            print('handle relation:', relation_name)
+            #print('handle relation:', relation_name)
             relation.entities = self.get_all(relation.foreign_model,
                                     filter_condition=filter_condition,
                                     orderby=orderby,
@@ -285,8 +299,8 @@ class QueryWizzard:
             #print('entities:', relation.entities)
 
     def load_one_to_many_entities(self, entity, one_to_many_options):
-        print('load_one_to_many_entities, options:')
-        print(one_to_many_options)
+        #print('load_one_to_many_entities, options:')
+        #print(one_to_many_options)
         if one_to_many_options is None:
             one_to_many_options = {'load_12m': True}
         else:
@@ -313,13 +327,13 @@ class QueryWizzard:
                 page_size = options.get('page_size', None)
                 active_page = options.get('active_page', None)
 
-            print('handle relation:', relation_name)
+            #print('handle relation:', relation_name)
             relation.entities = self.get_all(relation.foreign_model,
                                     filter_condition=filter_condition,
                                     orderby=orderby,
                                     page_size=page_size,
                                     active_page=active_page)
-            print(f'loaded {len(relation.entities)} entities')
+            #print(f'loaded {len(relation.entities)} entities')
 
     def include_many_to_many_join(self, model, relation, from_clause):
         # !! todo: first check if not already joined!!
@@ -341,7 +355,7 @@ class QueryWizzard:
             orderby=None, ascending=None, page_size=None, active_page=None):
         """filters and filter_condition cannot be both not None"""
 
-        print('\nbuild_get_all_query\n')
+        #print('\nbuild_get_all_query\n')
         qb = QueryBuilder(model=self.get_model(model), qw=self)
         return qb.build_get_all_query(
             select_fields=select_fields,
@@ -647,43 +661,40 @@ class QueryWizzard:
         )
         return query
 
-    def prepare_data(self, instance):
-        data = instance.data
-        prepared_data = dict(data)
-        model = instance.model
-        # todo: can substitute by same method as m2m fields below..
-        # no need for lookup, except not loaded yet
+    def prepare_m2m_data(self, model, prepared_data, instance=None):
+        for m2m_relation_name, m2m_relation in model.many_to_many_relations():
+            prepared_data[m2m_relation_name] = getattr(instance, m2m_relation_name).entities
 
-        #print('prepare_data')
-        #print(data)
+    def prepare_m21_data(self, model, data, prepared_data):
         for column, value in data.items():
             try:
                 relation_name, relation = model.get_relation_by_fk_id_column(column)
             except RelationNotFoundException as exc:
-                # log
-                print(str(exc))
+                # todo: log
+                # print(str(exc))
+                print('did not find relation ', column)
                 pass
             else:
-                # print(f'\n did find relation in column {column}')
                 if isinstance(relation, ManyToOne):
+                    print('handle m21:', relation_name)
                     # if columns not equal
                     # refactor: retrieve value by id
+                    relation_foreign_model = self.get_model(relation.foreign_model)
                     if relation.load_all:
-                        prepared_data[relation_name+'_all'] = self.get_all(relation.foreign_model)
+                        print('load_all')
+                        prepared_data[relation_name+'_all'] = self.get_all(relation_foreign_model)
                         print(prepared_data[relation_name+'_all'])
 
                     prepared_value = self.retrieve_value_by_value(
-                        model=relation.foreign_model,
+                        model=relation_foreign_model,
                         lookup_column=relation.update_search_column,
-                        filter_column=relation.foreign_model.id_field_name(),
+                        filter_column=relation_foreign_model.id_field_name(),
                         filter_value=value
                     )
                     prepared_data[relation_name] = prepared_value
                 else:
+                    # todo: log
                     raise Exception(f'Error during data preparation: Could not handle {relation}')
-        for m2m_relation_name, m2m_relation in model.many_to_many_relations():
-            prepared_data[m2m_relation_name] = getattr(instance, m2m_relation_name).entities
-        return prepared_data
 
     def retrieve_value_by_value(self, model, lookup_column, filter_column, filter_value):
         # todo generalize to search function
@@ -735,12 +746,12 @@ class QueryWizzard:
         return data
 
     def build_m2m_update_queries(self, model, instance_id, relation, actual_ids, new_ids):
-        print('update_m2m_relation')
+        #print('update_m2m_relation')
         model = self.get_model(model)
         foreign_model = self.get_model(relation.foreign_model)
 
-        print('new_ids:', new_ids)
-        print('actual_ids:', actual_ids)
+        #print('new_ids:', new_ids)
+        #print('actual_ids:', actual_ids)
         if isinstance(actual_ids, str):
             actual_ids = extract_ids(actual_ids)
         if isinstance(new_ids, str):
@@ -773,16 +784,34 @@ class QueryWizzard:
         # todo: delete in one query with 'IN' expression
         return queries
 
+    def delete_by_condition(self, model, filter_condition, commit=True):
+        model = self.get_model(model)
+        delete_query = DeleteQuery(model.table_name, filter_condition)
+        self.execute_query(delete_query)
+
     def create(self, model, data, commit=True):
-        # todo: make as mighty as update (relations, etc.)
         model = self.get_model(model)
         inserts = dict()
+        m2m_insert_queries = []
+
         for column, value in data.items():
-            print('column:', column)
+            # print('column:', column)
             field = model.get_field(column)
-            print('field:', field)
+            # print('field:', field)
             if field is None:
-                pass
+                if not value:
+                    continue
+                relation = model.get_relation(column)
+                if isinstance(relation, ManyToOne):
+                    if relation.load_all:
+                        fk_id = int(value)
+                    else:
+                        fk_id = self.retrieve_id_by_value(
+                            model=relation.foreign_model,
+                            filter_column=relation.update_search_column,
+                            filter_value=value
+                        )
+                    inserts[relation.foreign_key_field] = fk_id
             else:
                 # todo: refactor
                 # if isinstance(field, DateTimeField):
@@ -797,21 +826,49 @@ class QueryWizzard:
                 #         )
                 inserts[column] = value
 
-        print('inserts:', inserts)
+        #print('inserts:', inserts)
         insert_query = InsertQuery.build(
             table=model.table_name, inserts=inserts)
-        print('params of insert_query:', insert_query.params)
-        self.execute_queries_in_transaction([insert_query,])
+        #print('params of insert_query:', insert_query.params)
+        #self.execute_queries_in_transaction([insert_query,])
 
-    def delete_by_condition(self, model, filter_condition, commit=True):
-        model = self.get_model(model)
-        delete_query = DeleteQuery(model.table_name, filter_condition)
-        self.execute_query(delete_query)
+        try:
+            self.execute_query(insert_query)
+            inserted_id = self.last_insert_rowid()
+
+            for column, value in data.items():
+                field = model.get_field(column)
+                # print('field:', field)
+                if field is None:
+                    if not value:
+                        continue
+                    relation = model.get_relation(column)
+                    if isinstance(relation, ManyToMany):
+                        m2m_insert_queries.extend(
+                                self.build_m2m_update_queries(
+                                    model=model,
+                                    instance_id=inserted_id,
+                                    relation=relation,
+                                    actual_ids=[],
+                                    new_ids=value
+                                )
+                        )
+
+            for query in m2m_insert_queries:
+                self.execute_query(query)
+        except Exception as exc:
+            self.rollback()
+            raise exc
+        else:
+            self.commit()
+            print('successfully committed all queries in transaction')
+            return inserted_id
 
     def update(self, model, filter_condition, data, instance=None, commit=True):
         # todo only update changed data
         # i.e. difference data - instance.data
-        # todo: all in transaction!
+        # todo: what about commit?
+
         model = self.get_model(model)
         updates = dict()
         m2m_update_queries = []
@@ -837,12 +894,14 @@ class QueryWizzard:
                 elif isinstance(relation, ManyToMany):
                     # compare difference
                     # if instance is None, needs to be loaded first..
-                    m2m_update_queries.extend(self.build_m2m_update_queries(
-                        model=model,
-                        instance_id=instance.id,
-                        relation=relation,
-                        actual_ids=[entity.id for entity in getattr(instance, column).entities],
-                        new_ids=value)
+                    m2m_update_queries.extend(
+                        self.build_m2m_update_queries(
+                            model=model,
+                            instance_id=instance.id,
+                            relation=relation,
+                            actual_ids=[entity.id for entity in getattr(instance, column).entities],
+                            new_ids=value
+                        )
                     )
 
                 elif relation is None:
@@ -854,7 +913,7 @@ class QueryWizzard:
 
         update_query = UpdateQuery.build(
             model, filter_condition=filter_condition, updates=updates)
-        print('params of update_query:', update_query.params)
+        #print('params of update_query:', update_query.params)
 
         self.execute_queries_in_transaction(queries=[update_query] + m2m_update_queries)
 
@@ -897,4 +956,4 @@ class QueryWizzard:
         if isinstance(field, IntegerField):
             return 'INTEGER'
         if isinstance(field, DateTimeField):
-            return 'TEXT'
\ No newline at end of file
+            return 'TEXT'
